// SPDX-License-Identifier: GPL-2.1-or-later
/*
 * gp_pixel_channel.gen.h
 *
 * GENERATED on 2024 11 02 07:21:42 from gp_pixel_channel.gen.h.t
 *
 * DO NOT MODIFY THIS FILE DIRECTLY!
 */
#ifndef GP_PIXEL_CHANNEL_GEN_H
#define GP_PIXEL_CHANNEL_GEN_H

/*
 * Pixel channel operations.
 *
 * Copyright (C) 2011      Tomas Gavenciak <gavento@ucw.cz>
 * Copyright (C) 2013-2024 Cyril Hrubis <metan@ucw.cz>
 */

/**
 * @file gp_pixel_channel.gen.h
 * @brief A pixel channel manipulations generated from gen/include/gfxprim_config.py
 *
 * @warning All the actual values here depends on the configuration. If you
 *          change the definitions of pixel types in the gfxprim_config.py
 *          values will be different!
 */

#include <core/gp_clamp.h>

/**
 * @brief Macro to get raw channel value for RGB101010 pixel.
 */
#define GP_PIXEL_GET_R_RGB101010(p) (GP_GET_BITS(20, 10, (p)))
#define GP_PIXEL_GET_G_RGB101010(p) (GP_GET_BITS(10, 10, (p)))
#define GP_PIXEL_GET_B_RGB101010(p) (GP_GET_BITS(0, 10, (p)))

/*
 * macros to create gp_pixel of pixel type RGB101010 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGB101010(R, G, B) (0\
	+ ((R) << 20) \
	+ ((G) << 10) \
	+ ((B) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for xRGB8888 pixel.
 */
#define GP_PIXEL_GET_R_xRGB8888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_PIXEL_GET_G_xRGB8888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_PIXEL_GET_B_xRGB8888(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to create gp_pixel of pixel type xRGB8888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_xRGB8888(R, G, B) (0\
	+ ((R) << 16) \
	+ ((G) << 8) \
	+ ((B) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for RGBA8888 pixel.
 */
#define GP_PIXEL_GET_R_RGBA8888(p) (GP_GET_BITS(24, 8, (p)))
#define GP_PIXEL_GET_G_RGBA8888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_PIXEL_GET_B_RGBA8888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_PIXEL_GET_A_RGBA8888(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to create gp_pixel of pixel type RGBA8888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGBA8888(R, G, B, A) (0\
	+ ((R) << 24) \
	+ ((G) << 16) \
	+ ((B) << 8) \
	+ ((A) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for RGB888 pixel.
 */
#define GP_PIXEL_GET_R_RGB888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_PIXEL_GET_G_RGB888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_PIXEL_GET_B_RGB888(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to create gp_pixel of pixel type RGB888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGB888(R, G, B) (0\
	+ ((R) << 16) \
	+ ((G) << 8) \
	+ ((B) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for BGR888 pixel.
 */
#define GP_PIXEL_GET_B_BGR888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_PIXEL_GET_G_BGR888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_PIXEL_GET_R_BGR888(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to create gp_pixel of pixel type BGR888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_BGR888(B, G, R) (0\
	+ ((B) << 16) \
	+ ((G) << 8) \
	+ ((R) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for RGB555 pixel.
 */
#define GP_PIXEL_GET_R_RGB555(p) (GP_GET_BITS(10, 5, (p)))
#define GP_PIXEL_GET_G_RGB555(p) (GP_GET_BITS(5, 5, (p)))
#define GP_PIXEL_GET_B_RGB555(p) (GP_GET_BITS(0, 5, (p)))

/*
 * macros to create gp_pixel of pixel type RGB555 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGB555(R, G, B) (0\
	+ ((R) << 10) \
	+ ((G) << 5) \
	+ ((B) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for RGB565 pixel.
 */
#define GP_PIXEL_GET_R_RGB565(p) (GP_GET_BITS(11, 5, (p)))
#define GP_PIXEL_GET_G_RGB565(p) (GP_GET_BITS(5, 6, (p)))
#define GP_PIXEL_GET_B_RGB565(p) (GP_GET_BITS(0, 5, (p)))

/*
 * macros to create gp_pixel of pixel type RGB565 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGB565(R, G, B) (0\
	+ ((R) << 11) \
	+ ((G) << 5) \
	+ ((B) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for RGB666 pixel.
 */
#define GP_PIXEL_GET_R_RGB666(p) (GP_GET_BITS(12, 6, (p)))
#define GP_PIXEL_GET_G_RGB666(p) (GP_GET_BITS(6, 6, (p)))
#define GP_PIXEL_GET_B_RGB666(p) (GP_GET_BITS(0, 6, (p)))

/*
 * macros to create gp_pixel of pixel type RGB666 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGB666(R, G, B) (0\
	+ ((R) << 12) \
	+ ((G) << 6) \
	+ ((B) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for RGB332 pixel.
 */
#define GP_PIXEL_GET_R_RGB332(p) (GP_GET_BITS(5, 3, (p)))
#define GP_PIXEL_GET_G_RGB332(p) (GP_GET_BITS(2, 3, (p)))
#define GP_PIXEL_GET_B_RGB332(p) (GP_GET_BITS(0, 2, (p)))

/*
 * macros to create gp_pixel of pixel type RGB332 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_RGB332(R, G, B) (0\
	+ ((R) << 5) \
	+ ((G) << 2) \
	+ ((B) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for CMYK8888 pixel.
 */
#define GP_PIXEL_GET_K_CMYK8888(p) (GP_GET_BITS(24, 8, (p)))
#define GP_PIXEL_GET_Y_CMYK8888(p) (GP_GET_BITS(16, 8, (p)))
#define GP_PIXEL_GET_M_CMYK8888(p) (GP_GET_BITS(8, 8, (p)))
#define GP_PIXEL_GET_C_CMYK8888(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to create gp_pixel of pixel type CMYK8888 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_CMYK8888(K, Y, M, C) (0\
	+ ((K) << 24) \
	+ ((Y) << 16) \
	+ ((M) << 8) \
	+ ((C) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for P2 pixel.
 */
#define GP_PIXEL_GET_P_P2(p) (GP_GET_BITS(0, 2, (p)))

/*
 * macros to create gp_pixel of pixel type P2 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_P2(P) (0\
	+ ((P) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for P4 pixel.
 */
#define GP_PIXEL_GET_P_P4(p) (GP_GET_BITS(0, 4, (p)))

/*
 * macros to create gp_pixel of pixel type P4 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_P4(P) (0\
	+ ((P) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for P8 pixel.
 */
#define GP_PIXEL_GET_P_P8(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to create gp_pixel of pixel type P8 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_P8(P) (0\
	+ ((P) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for G1_DB pixel.
 */
#define GP_PIXEL_GET_V_G1_DB(p) (GP_GET_BITS(0, 1, (p)))

/*
 * macros to create gp_pixel of pixel type G1_DB directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G1_DB(V) (0\
	+ ((V) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for G2_DB pixel.
 */
#define GP_PIXEL_GET_V_G2_DB(p) (GP_GET_BITS(0, 2, (p)))

/*
 * macros to create gp_pixel of pixel type G2_DB directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G2_DB(V) (0\
	+ ((V) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for G4_DB pixel.
 */
#define GP_PIXEL_GET_V_G4_DB(p) (GP_GET_BITS(0, 4, (p)))

/*
 * macros to create gp_pixel of pixel type G4_DB directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G4_DB(V) (0\
	+ ((V) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for G1_UB pixel.
 */
#define GP_PIXEL_GET_V_G1_UB(p) (GP_GET_BITS(0, 1, (p)))

/*
 * macros to create gp_pixel of pixel type G1_UB directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G1_UB(V) (0\
	+ ((V) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for G2_UB pixel.
 */
#define GP_PIXEL_GET_V_G2_UB(p) (GP_GET_BITS(0, 2, (p)))

/*
 * macros to create gp_pixel of pixel type G2_UB directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G2_UB(V) (0\
	+ ((V) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for G4_UB pixel.
 */
#define GP_PIXEL_GET_V_G4_UB(p) (GP_GET_BITS(0, 4, (p)))

/*
 * macros to create gp_pixel of pixel type G4_UB directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G4_UB(V) (0\
	+ ((V) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for G8 pixel.
 */
#define GP_PIXEL_GET_V_G8(p) (GP_GET_BITS(0, 8, (p)))

/*
 * macros to create gp_pixel of pixel type G8 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G8(V) (0\
	+ ((V) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for GA88 pixel.
 */
#define GP_PIXEL_GET_V_GA88(p) (GP_GET_BITS(0, 8, (p)))
#define GP_PIXEL_GET_A_GA88(p) (GP_GET_BITS(8, 8, (p)))

/*
 * macros to create gp_pixel of pixel type GA88 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_GA88(V, A) (0\
	+ ((V) << 0) \
	+ ((A) << 8) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
/**
 * @brief Macro to get raw channel value for G16 pixel.
 */
#define GP_PIXEL_GET_V_G16(p) (GP_GET_BITS(0, 16, (p)))

/*
 * macros to create gp_pixel of pixel type G16 directly from given values.
 * The values MUST be already clipped/converted to relevant value ranges.
 */
#define GP_PIXEL_CREATE_G16(V) (0\
	+ ((V) << 0) \
	)

/*
 * macros to do per-channel operations on a pixel.
 */
#define GP_PIXEL_CHANS_ADD_RGB101010(pixel, perc) \
	GP_PIXEL_CREATE_RGB101010( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGB101010(pixel), 0x3ff * perc / 100, 0x3ff), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGB101010(pixel), 0x3ff * perc / 100, 0x3ff), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGB101010(pixel), 0x3ff * perc / 100, 0x3ff) \
	)

#define GP_PIXEL_CHANS_ADD_xRGB8888(pixel, perc) \
	GP_PIXEL_CREATE_xRGB8888( \
		GP_SAT_ADD(GP_PIXEL_GET_R_xRGB8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_G_xRGB8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_B_xRGB8888(pixel), 0xff * perc / 100, 0xff) \
	)

#define GP_PIXEL_CHANS_ADD_RGBA8888(pixel, perc) \
	GP_PIXEL_CREATE_RGBA8888( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGBA8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGBA8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGBA8888(pixel), 0xff * perc / 100, 0xff), \
		GP_PIXEL_GET_A_RGBA8888(pixel) \
	)

#define GP_PIXEL_CHANS_ADD_RGB888(pixel, perc) \
	GP_PIXEL_CREATE_RGB888( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGB888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGB888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGB888(pixel), 0xff * perc / 100, 0xff) \
	)

#define GP_PIXEL_CHANS_ADD_BGR888(pixel, perc) \
	GP_PIXEL_CREATE_BGR888( \
		GP_SAT_ADD(GP_PIXEL_GET_B_BGR888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_G_BGR888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_R_BGR888(pixel), 0xff * perc / 100, 0xff) \
	)

#define GP_PIXEL_CHANS_ADD_RGB555(pixel, perc) \
	GP_PIXEL_CREATE_RGB555( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGB555(pixel), 0x1f * perc / 100, 0x1f), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGB555(pixel), 0x1f * perc / 100, 0x1f), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGB555(pixel), 0x1f * perc / 100, 0x1f) \
	)

#define GP_PIXEL_CHANS_ADD_RGB565(pixel, perc) \
	GP_PIXEL_CREATE_RGB565( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGB565(pixel), 0x1f * perc / 100, 0x1f), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGB565(pixel), 0x3f * perc / 100, 0x3f), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGB565(pixel), 0x1f * perc / 100, 0x1f) \
	)

#define GP_PIXEL_CHANS_ADD_RGB666(pixel, perc) \
	GP_PIXEL_CREATE_RGB666( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGB666(pixel), 0x3f * perc / 100, 0x3f), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGB666(pixel), 0x3f * perc / 100, 0x3f), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGB666(pixel), 0x3f * perc / 100, 0x3f) \
	)

#define GP_PIXEL_CHANS_ADD_RGB332(pixel, perc) \
	GP_PIXEL_CREATE_RGB332( \
		GP_SAT_ADD(GP_PIXEL_GET_R_RGB332(pixel), 0x7 * perc / 100, 0x7), \
		GP_SAT_ADD(GP_PIXEL_GET_G_RGB332(pixel), 0x7 * perc / 100, 0x7), \
		GP_SAT_ADD(GP_PIXEL_GET_B_RGB332(pixel), 0x3 * perc / 100, 0x3) \
	)

#define GP_PIXEL_CHANS_ADD_CMYK8888(pixel, perc) \
	GP_PIXEL_CREATE_CMYK8888( \
		GP_SAT_ADD(GP_PIXEL_GET_K_CMYK8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_Y_CMYK8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_M_CMYK8888(pixel), 0xff * perc / 100, 0xff), \
		GP_SAT_ADD(GP_PIXEL_GET_C_CMYK8888(pixel), 0xff * perc / 100, 0xff) \
	)

#define GP_PIXEL_CHANS_ADD_P2(pixel, perc) \
	GP_PIXEL_CREATE_P2( \
		GP_SAT_ADD(GP_PIXEL_GET_P_P2(pixel), 0x3 * perc / 100, 0x3) \
	)

#define GP_PIXEL_CHANS_ADD_P4(pixel, perc) \
	GP_PIXEL_CREATE_P4( \
		GP_SAT_ADD(GP_PIXEL_GET_P_P4(pixel), 0xf * perc / 100, 0xf) \
	)

#define GP_PIXEL_CHANS_ADD_P8(pixel, perc) \
	GP_PIXEL_CREATE_P8( \
		GP_SAT_ADD(GP_PIXEL_GET_P_P8(pixel), 0xff * perc / 100, 0xff) \
	)

#define GP_PIXEL_CHANS_ADD_G1_DB(pixel, perc) \
	GP_PIXEL_CREATE_G1_DB( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G1_DB(pixel), 0x1 * perc / 100, 0x1) \
	)

#define GP_PIXEL_CHANS_ADD_G2_DB(pixel, perc) \
	GP_PIXEL_CREATE_G2_DB( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G2_DB(pixel), 0x3 * perc / 100, 0x3) \
	)

#define GP_PIXEL_CHANS_ADD_G4_DB(pixel, perc) \
	GP_PIXEL_CREATE_G4_DB( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G4_DB(pixel), 0xf * perc / 100, 0xf) \
	)

#define GP_PIXEL_CHANS_ADD_G1_UB(pixel, perc) \
	GP_PIXEL_CREATE_G1_UB( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G1_UB(pixel), 0x1 * perc / 100, 0x1) \
	)

#define GP_PIXEL_CHANS_ADD_G2_UB(pixel, perc) \
	GP_PIXEL_CREATE_G2_UB( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G2_UB(pixel), 0x3 * perc / 100, 0x3) \
	)

#define GP_PIXEL_CHANS_ADD_G4_UB(pixel, perc) \
	GP_PIXEL_CREATE_G4_UB( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G4_UB(pixel), 0xf * perc / 100, 0xf) \
	)

#define GP_PIXEL_CHANS_ADD_G8(pixel, perc) \
	GP_PIXEL_CREATE_G8( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G8(pixel), 0xff * perc / 100, 0xff) \
	)

#define GP_PIXEL_CHANS_ADD_GA88(pixel, perc) \
	GP_PIXEL_CREATE_GA88( \
		GP_SAT_ADD(GP_PIXEL_GET_V_GA88(pixel), 0xff * perc / 100, 0xff), \
		GP_PIXEL_GET_A_GA88(pixel) \
	)

#define GP_PIXEL_CHANS_ADD_G16(pixel, perc) \
	GP_PIXEL_CREATE_G16( \
		GP_SAT_ADD(GP_PIXEL_GET_V_G16(pixel), 0xffff * perc / 100, 0xffff) \
	)

#endif /* GP_PIXEL_CHANNEL_GEN_H */
